public class Parser {
    private static enum Action { Push, Pop }
    private static final Action[][] ACT;
    
    static {
        // ACT: an action table. based on the input token and the top symbol
        //      of the stack, what action needs to be taken.
        // Push: push the current input token to the stack
        // Pop: pop the operator from the stack and perform the operation
        // Undefined elements (null) mean an error

        ACT = new Action[128][128]; //[stack top][input token]
        
        //TODO: add table entries for unary minus '~' operator

        //stack top is '#'
        for(char c : "(+-*/".toCharArray())     ACT['#'][c] = Action.Push;
        for(char c : "$".toCharArray())         ACT['#'][c] = Action.Pop;

        //stack top is '('
        for(char c : "(+-*/".toCharArray())     ACT['('][c] = Action.Push;
        for(char c : ")".toCharArray())         ACT['('][c] = Action.Pop;

        //stack top is '+'
        for(char c : "(*/".toCharArray())       ACT['+'][c] = Action.Push;
        for(char c : "+-)$".toCharArray())      ACT['+'][c] = Action.Pop;

        //stack top is '-'
        for(char c : "(*/".toCharArray())       ACT['-'][c] = Action.Push;
        for(char c : "+-)$".toCharArray())      ACT['-'][c] = Action.Pop;

        //stack top is '*'
        for(char c : "(".toCharArray())         ACT['*'][c] = Action.Push;
        for(char c : "+-*/)$".toCharArray())    ACT['*'][c] = Action.Pop;

        //stack top is '/'
        for(char c : "(".toCharArray())         ACT['/'][c] = Action.Push;
        for(char c : "+-*/)$".toCharArray())    ACT['/'][c] = Action.Pop;
    }
    
    private static class Node {
        protected Integer   num;
        protected Character opr;

        public Node(Integer num)   { this.num = num; }
        public Node(Character opr) { this.opr = opr; }
        public String toString() {
            return num != null ? num.toString()
                :  opr != null ? opr.toString()
                :  ""
                ;
        }
    }
    
    public static BinaryTree<Node> parseExpr(String expr) {
        Scanner scan = new Scanner(expr + "$");
        Stack<LinkedBinaryTree<Node>> stack_tree = new StackByArray<>();
        Stack<Character> stack_oper = new StackByArray<>();

        //TODO: - parseExpr will be similar to evalExpr function that evaluates
        //        infix expressions.
        //      - Here, instead of using the operand stack, we push/pop subtrees of
        //        the parse tree to/from the tree stack.  
        //      - When popping an operator, pop one or two parse-trees from the tree stack; 
        //        build a parse-tree rooted at the operator; and push the resulting tree
        //        onto the tree stack        

    }

    public static double evalExpr(BinaryTree<Node> tree) {
        Stack<Double> num = new StackByArray<Double>();
    
        //TODO: - evalExpr will be similar to evalPostfixExpr function that evaluates
        //        postfix expressions.
        //      - While traversing the nodes of the parseTree in the post-order,
        //        evaluate the expression by pushing/popping operands to/from the stack num 

    }
    
    public static String infixToPrefix(String expr) {
        String strExp = "";
        BinaryTree<Node> parseTree = parseExpr(expr);
        for(Position<Node> p: parseTree.preorder())
            strExp += p.getElement() + " ";
        return strExp;
    }

    public static String infixToPostfix(String expr) {
        String strExp = "";
        BinaryTree<Node> parseTree = parseExpr(expr);
        for(Position<Node> p: parseTree.postorder())
            strExp += p.getElement() + " ";
        return strExp;
    }
    
    public static String infixToInfix(String expr) {
        String strExp = "";
        BinaryTree<Node> parseTree = parseExpr(expr);
        for(Position<Node> p: parseTree.inorder())
            strExp += p.getElement() + " ";
        return strExp;
    }
}
